/*
 *  mod_musicindex.h
 *  mod_musicindex
 *  
 *  $Id: mod_musicindex.h,v 1.20 2003/06/27 19:00:29 varenet Exp $
 *
 *  Created by Thibaut VARENE on Thu Mar 20 2003.
 *  Copyright (c) 2003 Regis BOUDIN
 *  Copyright (c) 2003 Thibaut VARENE
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1, or (at your option)
 *  any later version.
 *
 */

#ifndef MOD_MUSICINDEX_H
#define MOD_MUSICINDEX_H

#include <apache-1.3/httpd.h>
#include <apache-1.3/http_config.h>
#include <apache-1.3/http_core.h>
#include <apache-1.3/http_request.h>
#include <apache-1.3/http_protocol.h>
#include <apache-1.3/http_log.h>
#include <apache-1.3/http_main.h>
#include <apache-1.3/util_script.h>
#include <apache-1.3/util_uri.h>
#include <apache-1.3/fnmatch.h>
#include <apache-1.3/ap_ebcdic.h>
#include <apache-1.3/ap_alloc.h>

#include <stdlib.h>

module MODULE_VAR_EXPORT musicindex_module;

/**
 * Compile time configuration option
 */
#undef NO_TITLE_STRIP	/* Disable pretty printing for untagged files */


/**
 * Current version number & authors
 */
#define MUSIC_VERSION_STRING	"0.7.8"	/* Version string */
#define MUSIC_AUTHORS_STRING	"R. Boudin & T. Varene"	/* Authors */

#define SB_ALBUM	'A'		/* Sort by album */
#define SB_ARTIST	'B'		/* Sort by artist */
#define SB_DATE		'D'		/* Sort by date */
#define SB_FILETYPE	'F'		/* Sort by filetype */
#define SB_LENGTH	'L'		/* Sort by length */
#define SB_FILENAME	'M'		/* Sort by filename */
#define SB_TRACK	'N'		/* Sort by track */
#define SB_POSN		'P'		/* Sort by Part Of Set nb */
#define SB_BITRATE	'R'		/* Sort by bitrate */
#define SB_SIZE		'S'		/* Sort by filesize */
#define SB_TITLE	'T'		/* Sort by title */
#define SB_URI		'U'		/* Sort by URI */
#define SB_DEFAULT	'U'		
#define SB_RANDOM	'Z'		

#define FT_OGG		'O'
#define FT_MP3		'M'
#define FT_DIR		'Z'

#define D_ASCENDING	1		/* Sort ascending */
#define D_DESCENDING	-1		/* Sort descending */

#define MUSICINDEX	'I'
#define PLAYLIST	'P'

#define ARG_NUMBER	16		/* Number of args to sort lists */

#define MI_ACTIVE	1
#define MI_PLAYLIST	2
#define MI_ALLOWSTREAM	4
#define MI_ALLOWDWNLD	8
#define MI_PLAYALL	16
#define MI_RECURSIVE	32
#define MI_QUICKPL	64
#define MI_SEARCH	128
#define MI_ALLOWSEARCH	256

/**
 * Common reference for maximum string size.
 */
#define MAX_STRING	1024

typedef int (*pt2function) ();

typedef struct mu_ent_names {
	char *filename;
	char *uri;
} mu_ent_names;

typedef struct mu_ent {
	char *file;
	char *uri;
	char *album;
	char *artist;
	char *title;
	unsigned short date;
	unsigned short track;
	unsigned short posn;
	unsigned long length;
	unsigned long bitrate;
	unsigned long size;
	char filetype;
	struct mu_ent *next;
} mu_ent;

typedef struct {
	char order[ARG_NUMBER];
	char fields[ARG_NUMBER];
	pt2function order_functions[ARG_NUMBER];
	
	char *cd_icon;
	char *small_cd_icon;
	char *sound_icon;
	char *fetch_icon;
	char *arrow;
	char *css;
	char *search;

	unsigned short play_recursive;

	unsigned short options;
} mu_config;

struct mu_ent *new_ent(pool *pool, struct mu_ent *head);
struct mu_ent *quicksort(struct mu_ent *base, struct mu_ent *end,
	mu_config * conf);
int handle_musicindex(request_rec * r);

#endif
